
#This script will create copies of each .MSP file in the directory replacing all instances of illegal
#strings for charge states and modifications to make them compatible with SpectraST library importer.
#KNOWN ISSUE
#SpectraST importer will still refuse any spectra with more than one modification on the same position.
#version 2.0 2018/07/07
try {
    $path = $PSScriptRoot
    $files = Get-ChildItem $path -Filter *.msp | Where-Object {-not $_.FullName.contains('_fix.msp')}
    Write-Host "This will create copies of the following MSP libraries:"
    For ($i = 0; $i -lt $files.count; $i++) {
        Write-Host ($files[$i].FullName.Split("\")[-1])
        }
    $pro = Read-Host -Prompt '[Y] Yes  [N] No  (default is "N")'
        if(-not ($pro -in @('Y','y'))){
            return
        }
    For ($i = 0; $i -lt $files.count; $i++) {
        $infile = $files[$i].FullName;
        $infileshort = $files[$i].FullName.Split("\")[-1];
        Write-Progress -Activity "Processing files in '$path'" -percentComplete ($i / $files.count * 100) -Id 1
    
        $inWithoutExt = [System.IO.Path]::GetFileNameWithoutExtension($infile);
    
        $outfile = $path + "\" + $inWithoutExt + "_fix.msp";
        $original = [System.IO.Path]::Combine($path, $infile);
        $mode = [System.IO.FileMode]::Create
        $access = [System.IO.FileAccess]::Write
        $sharing = [IO.FileShare]::Read
        try {
            $fs = New-Object IO.FileStream($outfile, $mode, $access, $sharing)
            $stream = New-Object -TypeName System.IO.StreamWriter($fs, [System.Text.Encoding]::UTF8);
            $fileSizeInput = (Get-Item $original).length
            $counter = 0
            foreach ($line in [System.IO.File]::ReadLines($original)) {  
                if ($line -match "^Comment.*") {
                    $line = $line.replace(' Charge=', '/');
                    $line = $line.replace(',CAM', ',Carbamidomethyl');
                    $line = $line.replace(',iTRAQ', ',iTRAQ4plex');
                }
                $stream.WriteLine($line);          
                $counter++
                if ($counter % 100000 -eq 0) {
                    $counter = 0
                    $stream.Flush()
                    $fileSizeOut = (Get-Item $outfile).length;
                    Write-Progress -Id 2 -Activity "Processing file '$infileshort'" -percentComplete ($fileSizeOut / $fileSizeInput * 100) -ParentId 1
                }
            }
            Write-Progress -Id 2 -Activity "Processing file $infileshort" -Completed
          
        }
        finally {
            $stream.Dispose()
            $fs.Dispose()   
        }
    }
    Write-Progress -Activity "Processing files in $path" -Id 1 -Completed
}
catch {
    Write-Error $Error[0].ScriptStackTrace 
}
finally {
    Write-Host "Finished fixing MSP Libraries"
    pause
}